﻿using Adaptation.Shared;
using Adaptation.Shared.Properties;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Adaptation.FileHandlers.MET08THFTIRQS408M;

public class WSRequest
{

    public long Id { get; set; }
    public string Batch { get; set; }
    public string Cassette { get; set; }
    public string CellName { get; set; }
    public string Date { get; set; }
    public string FilePath { get; set; }
    public string MeanThickness { get; set; }
    public string Operator { get; set; }
    public string PSN { get; set; }
    public string RDS { get; set; }
    public string Reactor { get; set; }
    public string Recipe { get; set; }
    public string StdDev { get; set; }
    public string Title { get; set; }
    public string UniqueId { get; set; }
    public List<Stratus.Detail> Details { get; protected set; }

    [Obsolete("For json")] public WSRequest() { }

    internal WSRequest(IFileRead fileRead, Logistics logistics, List<Stratus.Description> descriptions)
    {
        Id = 0;
        FilePath = string.Empty;
        if (fileRead is null)
        { }
        CellName = logistics.MesEntity;
        if (descriptions[0] is not Stratus.Description x)
            throw new Exception();
        Details = new List<Stratus.Detail>();
        //Header
        {
            Batch = x.Lot;
            Cassette = x.Cassette;
            Date = x.Date;
            MeanThickness = x.MeanThickness;
            Operator = x.Employee;
            PSN = x.PSN;
            RDS = x.RDS;
            Reactor = x.Reactor;
            Recipe = x.Recipe;
            StdDev = x.GradeStdDev;
            Title = x.Title;
            UniqueId = x.UniqueId;
        }
        string[] segments;
        Stratus.Detail detail;
        foreach (Stratus.Description description in descriptions)
        {
            detail = new Stratus.Detail
            {
                HeaderUniqueId = description.HeaderUniqueId,
                Mean = description.Mean,
                PassFail = description.PassFail,
                Position = description.Position,
                Recipe = description.Recipe,
                Slot = description.Slot,
                StdDev = description.StdDev,
                Thickness = description.Thickness,
                UniqueId = description.UniqueId,
                Wafer = description.Wafer,
            };
            detail.Points = new();
            segments = description.Position.Split(',');
            foreach (string segment in segments)
                detail.Points.Add(new Stratus.Point { HeaderUniqueId = description.HeaderUniqueId, UniqueId = description.UniqueId, Position = segment });
            segments = description.Thickness.Split(',');
            if (detail.Points.Count != segments.Length)
                throw new Exception();
            for (int i = 0; i < detail.Points.Count; i++)
                detail.Points[i].Thickness = segments[i];
            Details.Add(detail);
        }
        if (Date is null)
            Date = logistics.DateTimeFromSequence.ToString();
        if (UniqueId is null && Details.Any())
            UniqueId = Details[0].HeaderUniqueId;
    }

}